'use strict';


const fs = require('fs');
const config = require('../../config.js');

const fetch = (...args) => import('node-fetch').then(({default: fetch}) => fetch(...args));

async function chatGPT(id, message){

    let result;
    let max_tokens;

    if (/@kite\.wildix\.com/.test(id) ){
        max_tokens = config.chatGPT.externalmaxtokens;
    } else if (/@*wildixin\.com/.test(id)) {
        max_tokens = config.chatGPT.internalmaxtokens;
    }

    let response = await fetch('https://api.openai.com/v1/chat/completions', {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json',
            'Authorization': config.chatGPT.authorization,
            'Organisation': config.chatGPT.organization
        },
        body: JSON.stringify({
            'model': config.chatGPT.model,
            messages: [
                { role: 'system', content: config.chatGPT.systemcontent },
                { role: 'user', content: message }
            ],
            'temperature':  config.chatGPT.temperature,
            'max_tokens': max_tokens,
        })
    });

    let data = await response.json();

    console. log(data);
    console. log(response);

    if (data.hasOwnProperty('choices') && data.choices.length > 0) {
        const content = data.choices[0].message.content;
        console.log("Content:", content);
        result = content;
    } else {
        console.error('No valid choices in the response');
        result = 'Error. Please try again!';
    }
    console. log("result is: "+ result);

    return result;
}


class Responder {

    constructor(config) {
        this._workinghours = config.workinghours;
        this._nonworkinghours = config.nonworkinghours;
        this._contact = config.contact;
        this._sales = config.sales;
        this._support = config.support;
        this._accounts = config.accounts;
    }

    hello(id, userdata) {
        return this._processHello(userdata);
    }

    say(id, message) {
        return  this._processSay(id, message);
    }

    bye(id) {
    }

    _processHello(userdata) {
        const filename = this._workingHours() ? this._workinghours : this._nonworkinghours ;
        const content = fs.readFileSync(filename).toString();
        return this._processTemplate(userdata, content);
    }

    async _processSay(id, message) {
        const contact = this._contact;
        const sales = this._sales;
        const support = this._support;
        const accounts = this._accounts;
        let content;


        if (message == 'contact') {
            content = fs.readFileSync(contact).toString();
        }

        else if (message == 'sales') {
            content = fs.readFileSync(sales).toString();
        }

        else if (message == 'support') {
            content = fs.readFileSync(support).toString();
        }

        else if (message == 'accounts') {
            content = fs.readFileSync(accounts).toString();
        }

        else {

            content = await chatGPT(id, message);

        }

        return content;
    }




    _processTemplate(userdata, content) {
        let result = content.replace(/\$username/g, userdata.name);
        if(userdata.email) {
            result = result.replace(/\$useremail/g, userdata.email);
        }
        return result;
    }

    _workingHours() {
        const now = new Date();
        const day = now.getDay();
        const hour = now.getHours();
        return (day < 6) && (hour > 8 && hour < 14);
    }

}

module.exports = Responder;